//+-------------------------------------------------------------+ 
//|   Multicolor PNF Charting, V2.1 Demo, by LaurV, Sep 2007    |
//+-------------------------------------------------------------+ 
#property copyright "Copyright (C) September 2007, by LaurV"
#property link "forum.vamist.com"
//look for tradelover user on the forum, you can register and let me
//a PM there if you want to talk about the indicator or comment some
//of the stuff below. Any idea is welcome.....

// Version 2.1 - Sept 20, 2007
// ===========
// Please read below the description for V2.0, as that was written
// first (in time), during the developement process. That will help you
// understand what we are talking about here, with this new release. 
//  
// - All the story is the same as V2.0, but we added painting of the history
//   boxes. Please read carefully all the story below, and disregard the
//   part where we talk about "why the history was not painted in V2.0", as
//   we now solved the problems. The history is NOT painted using the lower
//   timeframes from the past, but as far as the OHLC prices are available, 
//   it should be accurate in the near past and the accuracy should "fade"
//   as we go further in the past, please read the comments below (V2.0)
//   related to this subject. We paint the reversal first, with a higher 
//   priority than the continuation, so even this is not mathematically 100%
//   correct, and it can generate some small fluctuations on the PF-grapfic
//   sometime, however in real live it is working very well, because as a
//   rule of thumb, getting out of a bad tranzaction is more important then
//   initiating a new one. And generally you must be carefull to correlate
//   the box size with the time frame. A wise trader will never trade W1
//   chart with a 10-pip box, or a M1 chart with 100-pip box. If the 
//   settings are right (for TF and box size) the history should look well,
//   almost perfect. You can have some additional boxes (return points) 
//   on the history, that should not be there, but all the supports and 
//   resistances are mathematically correct, and they will be drawn right, 
//   in any situation. Therefore in most of the cases, a normal trader can't
//   spot the small pullback fluctuation boxes, even if they are present
//   with the history. Anyhow is 100 times better than no history at all! :P
//   Remark: when you change the TF on the fly, the history could look a bit
//        different because of these pullbacks. In real time the PF graphic
//        must look the same, regardless of the TF you run it, as PF
//        charting is time-independent, and only box-size dependent. 
//   Tip: Big differences [in appearence of the graphic when you change the
//        TF on the fly, except the colors, which will always be different)], 
//        expecially for the near past period (close to rightmost candle) are
//        telling you that you have to use a bigger box if you want to have 
//        chances to make any profit...
//
// - New feature: box description. If you stay with your mouse on a box, 
//   then you could see when the box was created (the Time[] of the candle
//   that created the box). This is a very much welcomed feature, because
//   usually you see on screen (without scrolling, on a H1 chart, for example)
//   just few hundred candles from the last 10 or 20 days, but the M-PF boxes
//   cover (on the same screen) the price action for few years, or even all 
//   the history, with a convenient box size chosen. So, now you can pick a 
//   box with your mouse and check it's description (look for "CandleTime"
//   row in the yellow popup box and read the date), then you can go back in
//   time (scroll left) to the candle that produced the box and verify the
//   price acction more accurate. 
//   Remark: You must turn off "show object description" from the chart's
//           properties, otherwise your screen will get... busy and fussy :D
//
// - Updated feature: potential profit calculus for history included. 
//   The displayed profit will usually be lower than the "real" profit, see
//   the comments at the end of the start() function for the explanation.
//   There is also a gif in the zip that shows this point. The accuracy of
//   the history pays it's tribute too.
//

// Background and history (created with V2.0)
// ======================
//
//Modified from an old one, written by a russian guy. That one gave me the
//idea about arranging the boxes, so I started working on it directly on
//the source file, until it was nothing left, then the new born indicator
//was much better, and in respect with PNF theory. To match PNF rule, the
//reversing bar must be about 3 times bigger than the continuation bar, 
//the theory was completely ignored by the russian guy before me.
//I also introduced many new features in the indicator:
// - new ReversingBox parameter (can be set about 3 times TrendingBox)
// - substituted russian undisplayable comments from the original version
// - customizable colors. It was not possible to change colors in the original
//   version previous V2.0. Now is possible, but not advisable if you use 
//   the auto-coloring function, described below
// - auto-coloring function (my original idea, with all the calculus
//   and arrangements, THIS IS HOT! For the best visual result, the chart
//   can be skewed vertically until the boxes look squares - use the mouse
//   on the price axis and drag it down)
// - features for translation, scalling, combining PNF's (more M-PF with 
//   different parameters can work on the same chart)
// - solve "running left" problem
// - solve lagging problem
// - deinit() function 100 times faster (it had a "for" to 99999 or so)

// Version 2.0 Full
// ================
// - renamed to M-PF because of "multicolor" charting features introduced
// - algorithm for auto-compute the box height according with the history
// - algorithm to find B-rays and draw them as trendlines (see below)
// - algortihm to identify and mark B-Flags by changing their border color
//   to yellow (later on, color introduced as parameter)

// Version 2.0 Demo
// ================
// - I found an ugly bug (misconception) in painting the indicator for the 
//   history candles. I could not solve it fast, as the stuff seems to be 
//   very complicate and I need to rethink all, to look into other charts
//   with a shorter TF, and make interpolation too... GRRR....(see below
//   at the FAQ section). So, I decided to create a "Demo" version in which 
//   the indicator is painted only in real time. If you need the history,
//   then you have to use the workaround with the tester in visual mode, 
//   as described below.
// - as the "demo" was working too nice :P, some other features were also
//   removed from it. They are too good to be distributed for free....
// - calculus for potential profit (displayed as comment) substitutes the 
//   iCustom functionality

// In fact, here is nothing left from the old one, except the word
// "Body" in the name of the objects, and the idea with the frames around
// the boxes, that made a very nice visual effect and helped me to mark 
// the B-Flags, everything else was changed, improved, bug cleaned and/or  
// removed, this is a complete new indicator for PF charting that does not
// get lost to the left side, does not lose bars, highs, lows, does 
// not repaint the past, etc. And it can also make profit!
//
// Note: What you see here is the DEMO version. 
// ===========================================
//
// Demo version does NOT include the features to compute the box height 
// automatically according with the history, does NOT draw the binding rays
// for the bright-boxes, does NOT mark the B-Flags, and does NOT include the
// memory buffer to generate signals to be used by an EA (with iCustom function)
// for real time trading. Also, the biggest drawback of the current version
// is that it will NOT generate X/O columns for the history. It starts from
// present candle only, and continues in real time or visual testing time 
// (as long as the chart is feed with TICKS data by the broker's server or
// by the Strategy Tester program available in MT4)

// If you need all the features discussed above, most probably that would 
// cost you some money.. :D (even if I did not decide yet to sell it or to
// keep it for myself :D)

// *** Demo version can be freely distributed as long as include these comments
// *** and the files and photos attached in the current zip file

// FAQ section
// ===========
// Q. What is a PF chart? (sometimes called PNF or P'nF, Point and Figure)
// A. For that, you can read thousands of pages on web (investopedia or else)
//    Look for Renko charting (paticular case of PF) too. Generally,
//    PF-charting is a charting method, beside of candlesticks, bars, etc
//    that keeps record of the price regardless of time. There are also some
//    very good books about this subject. PF is not Holy Grail, but it can
//    certainly help you to make money in Forex, by itself!

// Q. What is a bright-box, what is auto-color?
// A. This is a completely new concept developed by me. Instead of using only
//    X and O to represent the columns/trends (i.e. up/down, green/red, etc.),
//    I was starting long ago in the past to use "more" letters. The idea
//    was to measure the "speed" of the trend too, together with the direction,
//    and to get the congestion zones and/or exhaustions of trends. It is a big
//    difference between an X-box where the price is sharply rising 50 points
//    and an X-box where the price oscillates 10 points up/down for a week,
//    banging his head on the box's walls. So I was starting to mark with "star" 
//    instead of X, or "filled O" instead of O, such kind of movements. In time
//    I found them quite usefull. First of all, the *'s tend to accumulate on
//    certain support/resistence levels, where they will certainly show congestion
//    levels and reversing points. Also, if you look to any column of X or O that
//    is enough long (let's say over 10-12 X/O's), which folows after a breakout, 
//    there will be very easy to see that you can find at least 2-3 stars or 
//    black dots inside it, and they will be situated exactly at Fibo 38.2, 61.8 
//    or other Fibo major levels. Later on, I started using more signs, as dots,
//    dashes, different letters, etc, and all culminate with using colors. 
//    In this version of the indicator, the PF chart is painted by default
//    with "very dark" red and green colors. You can change them, but my advice
//    is: DON'T do it, except when you turn auto-coloring OFF. If auto-color is 
//    turned ON (default) then the boxes start to get brighter and brighter in 
//    time, if the price is oscillating for a long period inside of the same box,
//    until they will get yellow or eventually completely white, as shown in the
//    attached photos. They shine (glow, or "brighten"), signaling you that the 
//    price was making big efforts to get out of the box. How do you use this
//    information? Depends on you.... See below some tips.

// Q. What is a B-ray? Can I do them by hand?
// A. For every box which is "enough bright", we look for other bright boxes or
//    turning points (end of X-columns and start of O-colums or viceversa) around
//    it, and we connect these with trendlines or with level lines. So-generated
//    trendlines work very nice as support/resistance levels and they will delimit
//    areas where the market tends to go congested. Breaking of a strong support
//    or resistance level has almost everytime a "bright star" at the breaking
//    point. That is because a "big fight" between bears and bulls will develop
//    each time if the price is comming close to the edges. Breaking of a B-ray is 
//    almost everytime a sign that a big move in that direction will follow. 
//    "Strength" of a B-ray is given by how many "stars" or "turning points" stay 
//    on it, same as for support/resistance/pivot lines in clasical theory. 
//    Of course, for the Demo version you can draw all this lines by yourself. 
//    This has the advantage you can decide wich line is important and which not.
//    The disadvantage is that all your trendlines will run away in time. Why? 
//    That is very simple: You can draw these lines on the PF chart only, because
//    there is no way to make an association "backward" between a PF-box and the 
//    candlestick that generated that box. Because the PF graphic does not depend
//    on the time, it will not shift to the left with every new candle (time) but
//    it will shift one position for every reversal of the trend. So, it does not
//    have the same moving speed as the candlestick graphic. Given a box, even for
//    the program will be very complicate find where is the candlestick that
//    produced that box. It can be far away in the past, lost from screen, or even
//    lost from the chart (MT4 can store a limited number of candles in charts). 
//    That is why the program (Full version) will draw the rays on the PF graphic
//    and NOT on the candlestick graphic. If the line is drawn by the program, then
//    he "knows" it, and he will "take care" of it (the ray is on his list of 
//    personal-created objects). Because MT4 connects the trendlines to the 
//    "time/price" parameter, all trendlines drawn by you (that are unknown for 
//    the program) will move left together with the candlestick chart, one position
//    for every new candle, and they will get lost from the PF diagram, and from the 
//    screen, as you know, unless you have a monitor that can display a WWWWXGA 
//    resolution :D. If you draw the rays by yourself, the program can not "select"
//    which ray to be asociated with PF and which ray to be allowed to move left.
//    However, you can use a trick: rename the trendline to "B-RayTTTTTXXXXXYYY" 
//    where TTTT is the time when the indicator was launched, XXXXX is the personal
//    number of the indicator (see how boxes are named) and YY is the number of the
//    trendline. In this way you are "cheating" the program to believe that the 
//    trendline is his own creation. To rename a line, select it with double click, 
//    then right-click on it and choose "properties". Then rename it carefully.
//    The "complicate" method of naming objects is designed for the case when you 
//    have more M-PF indicators with different box-sizes, on the same chart, they 
//    shouldn't get confused each other about the ownership of the objects on the
//    screen, so DO NOT modify the naming scheme, if you don't know exactly what 
//    you're doing! Generally, it would be better if you don't modify anything. :P
//    Thendlines created by the program will have the right name, therefore they 
//    will stay on the PF graphic (and won't move left with the candlesticks). 
//    Tip: you won't need to rename horizontal (level) lines, they will stay
//    at the same price regardless where the graphic is going, but you MUST 
//    rename the trendlines to have them fixed on the PF-chart. The easyest method
//    to rename a line without making mistakes is to select any box first, go 
//    to it's properties (double click) and then copy the name of the box using
//    CTRL+C. Then select your line, double click it, paste the name with CTRL+V
//    and then edit the "Body" part into "B-Ray". The numbers don't need to be
//    consecutive, but only need to have the right format, so you shouldn't bother
//    to edit them. Just be carefull not to have 2 lines with the same name. Pick
//    a different box every time, and your life will be much easier. Somehow....
//    Note: When a new candlestick or bar is created, i.e. after every new candle
//    is "born", the lines WILL be moved to the left one position. And ALL objects,
//    including the PF boxes, candles, bars, trendlines, MA's, RSI, labels, bla bla,
//    (except only the text object, that are anchored on the screen and not on the 
//    graphic) WILL ALL be moved to the left, with every new candle "born" on
//    the right side, this is internal kitchen of MT4 and can not be avoided, but 
//    the M-PF indicator shall detect the new candle and shall move all his own
//    objects back one position to the right. This is the "trick" that lets you
//    see the PF-boxes "fixed" on the screen when the candles fly to the left. 
//    The left/right movement is so fast that you can not see it, or you can see
//    it sometime as a slight "tremor", when your computer is busy). This is 
//    normal for all indicators and expert advisors.

// Q. What is a B-Flag? 
// A. The explanations here are given by example. You should take a paper and a
//    pencil to figure out all the story with raising X's and falling O's columns.
//    Sometime you will see that on the PF graphic there is not so much movement
//    compared with the candles that fly to the left in hurry. There was a raise
//    far away in the past (uptrend) and you have a green short column on your PF
//    chart, for example 3-5 signs of X, dark-colored green boxes, because the
//    uptrend was fast, but the candles that produced the move are not seen
//    anymore on the screen. Since two weeks the price is moving 10 points up and 
//    11 points down for hundred times, in such a way that a red PF box appeared
//    after the short green column, and the red box is brightened in time as the 
//    price is "stuck" inside and it can not fall back. The red box is getting hot
//    because the energy produced inside of it by the price which is banging his
//    head against the walls. >:)). The pattern described here looks like a small
//    flag with a dark pole and a golden shining fabric. There are big forces 
//    that support the price from falling. If you could go back to the candles
//    that produces this flag, you could find there was a "flag" in respect of the
//    classical concept of technical analysis, i.e. the price rose a lot in the 
//    beginning, with a straight-up move, followed by a zig-zag-zig-zag movement
//    with short lines, going slightly down, or followed by a small descending
//    triangle or pennant. The red PF-box in this case is not a signal of trend
//    reversal, as any other red boxes that follows a green column, but instead,
//    it is a STRONG signal of trend continuation. Most probably a break will
//    follow upwards, and the price will "reach the sky"... :D
//    Everything is valid for downside too (flags with dark red stick and bright
//    green sparkling fabric, that will signal the continuation of the downtrend, 
//    you can call them reversed B-Flags if you like... I don't care...)
//    So we decided to insulate the PF bright-flags and mark them on the chart 
//    by changing the border color of the boxes. On demo version, you can try 
//    to spot the flags by yourself. It will be nice exercise for you as a trader
//    "under developement". Try spotting them and trade them on backtests or demo
//    before going to trade them live, because this job is a "tricky" one !!
//    Note: The concept of B-flag is the same as the bright boxes in a long trend
//    (discussed above). Instead of a long column with some bright boxes inside, 
//    we have some "rebel" boxes where the retracement was a bit too long and 
//    created a new PF-column with reversed direction, but after the enthusiastic
//    contrarians will calm down, the trend will resume to his initial direction.
//    In this way we can say that B-Flag is an extension of B-Boxes to more than
//    one column (B-Flags take 3 columns). Imagine that you had set a higher value
//    for TrendingBox parameter. For example doubled. In this case, with the old 
//    (smaller) value for box size, we suppose you had a succession (in time) of
//    XXXXXOXXXXXXXX boxes (3 columns, where the first 2 columns form a flag, 
//    then the trend continues up). With the new value (doubled) you will get just
//    a single column of 6 X's, every two X's from above will be "compressed" into
//    one, with a doubled height, so the first 2 new X will "eat" 4 of the old X
//    above, the 3rd new X will "digest" the "XOXX" from above, and the last 3
//    new X's will come from the last 6 old X's above. If you add brightness to 
//    the new XXXXXX column, what you will get? Of course, the third X will be
//    brighter, because the price knocked the walls some more times. You have a 
//    shining star in the middle, at (about) 38.2% of the trend: XX*XXX. This 
//    example is "cosmeticized" on purose, to show you that "bright boxes" are
//    a very BIG concept: they "pack" the price evolution. Having a B-Flag or a 
//    B-Box on the trend is just a matter of box size. If you see an uptrend that
//    looks like xxx*xxxx*xxx, than you know that in a different "timeframe"
//    (i.e. for PF charting this means a different box size, as the time does not
//    matter here), your trend could look like xxxxoxoxxooxxxxxxxxooxxxooxxxxxxxx.
//    This is 11 columns, with 3 small flags inside. You just "unpacked" the trend
//    same way as you "unpack" D1 (daily) chart by changing the TF to H4. The HUGE
//    advantage here, is that you don't need to do this unpacking process on screen
//    by taking a smaller box size and repainting the indicator. You can do it at 
//    once in your mind, just looking at the colors and their brightness. You don't
//    need to read more timeframes in the same time, and in time, you will learn
//    how to get the full atvantage of this system, at first sight. Just a fast
//    look to the multi-colored PF chart, and you know if it is the time to trade
//    or the time to stay on the fence. You can extend these concepts to more
//    columns, as 5, 7, etc. This is too complicate to program and it will never
//    be implemented here. And, in fact, except the "brightness", this WAS the
//    KERNEL of PF Trading system from the early stage of it's design: you try
//    to spot the flags and reversed flags, draw pivots and wait for breakouts.
//    PF charts are all looking like flags: one long column up, then plenty of 
//    short up/down columns that generate supports, resistances, pivots, then a
//    breakout that generates a new flag (in the same direction or viceversa). 
//    All the chart is in fact a chain of few big "super-flags", where the poles
//    (poles can be on more columns too!) shows the major trend, and the "army" 
//    after it (the "fabric" really looks like strings of solders, nice aligned)
//    shows the period of "relaxing" after. There is a time to buy, a time to sell,
//    and a time to relax.... 40% of the market time, or even more, the price is
//    not trending at all, but is ranging, congesting, exhausting or... relaxing.
//    Your "goal" as PF trader would to be to spot the trends, the reversals, 
//    and first of all, the breakouts from such "super-flags". If you catch a LONG
//    column from its early stage and stay on it until first sign of reversal 
//    (until the column is finished) then you worth the money... If you can not 
//    do this, then stay on the fence! Or trade using different methods.
//    But this is not the place to teach you to use PF charts. See some books...
//    We did not do anything special here, just added some "brightness" to the 
//    small flags, and we won't ever pretend that we are able to spot big flags
//    automatically. Bright flags give stronger signals then "normal" flags, 
//    because of the market-forces involved, so, because we were not able to 
//    spot bigger flags, we added brightness to small ones :D. This compensates
//    our lack of programming skill and time :P
//    This method is not 100% proof, and more than that, it is not foolproof.
//    If it would be, or if we would be able to auto spot the super-flags, then
//    we would be in Hawaii since long time, and won't show anything to you! :P
//    Masters of PF charting can spot the breakouts at first sight and trade them
//    with profit. But all of us, including you, are far away to be such masters.
//    So, use any theory in this text on your own risk and only after a lot of
//    backtesting and demo trading.
    
// Q. How can I use the demo version to place tranzactions using an EA?
// A. Well... in fact you can't get any Buy or Sell signal from this indicator. 
//    The demo version does not use the indicator's buffers to store any info
//    related to the rays, flags, brightness, or other properties of the boxes
//    placed on the screen, so you can not call the iCustom() function from your 
//    EA to get this properties. You still can call it, but it will return you 
//    some "meaningless" information. Well.. not so meaningless... You still can
//    use this information if you design all the logic with colors, flags and rays
//    in your expert. The indicator will return next values, formated as a 
//    double (float) number:
//
//    0.0 - anytime after a new candle was born on the chart and there was no box
//        added to the screen (this is usually beginning of long candles; all short 
//        candles that won't escape the current box; etc). You can use this as a
//        "new candle" indicator, but be carefull that sometimes a GAP will occur
//        and one or more new PF boxes will be painted on the screen at the very
//        first tick of the new candle. In this case your EA will miss the candle.
//        If you need an indicator for a new bar or an expert to signal you when a
//        new candle is born (many people are missing this, and all the programs
//        I saw are just scrap, trying to test if the volume is zero or if the 
//        time is sharp, most of them are giving false positives and most of them
//        are missing candles when the market goes crazy), then check my posts on 
//        vamist.com forex forum, I had posted one, long ago.
//
//    1.0 - trend was just reversed up (one green box was last painted after one
//        or more red boxes were painted in the past)
//
//    -1.0 - trend was just reversed down (one red box was last painted after one 
//        or more green boxes were painted in the past)
//
//    2.0 - uptrend continuation (one green box was last painted after one or 
//        more green boxes were painted in the past)
//
//    -2.0 - downtrend continuation (one red box was last painted after one or 
//        more red boxes were painted in the past)
//
//    That is all, you can not get other info from the demo version using an EA. 
//    Of course, you can compute the number of boxes by reading the prices,
//    knowing the box size, or you can count the candles to guess the brightness
//    of the boxes, or you can read the color directly on screen by reading the 
//    object properties, or draw any ray you like, but you have to do it by yourself
//    inside of the EA that you design

// Q. What is the "profit" line that appear on the comment area on the upper left
//    part of my screen, and how can I use it?
// A. To compensate cutting off the buffers, discussed above, I introduced a 
//    "tentative" of profit calculus into the Demo version. This will compute and 
//    display on screen the potential profit of the indicator, given the pair, 
//    TrendingBox and ReversingBox parameters. To compute the profit, I consider
//    a very simple strategy: initiate a tranzaction after every confirmed reversal
//    and get out of the tranzaction when any sign of reversal. Supposing a column
//    of (one or more) green boxes (uptrend) was painted in the past, and then the
//    indicator just painted a red one. This could be a sign that the trend will 
//    reverse. In this case I will wait for a second red box to appear on screen 
//    and if it appears, then I "go short". Symmetrical for bullish reversal. 
//    I will "close" any "open" tranzaction if a box with different color appears, 
//    and I will use the "opening/closing" price to compute the "realized" profit. 
//    Please pay attention to the quotes above. There is no trading taking place!!
//    I only compute the "potential" profit. As you can not use this indicator
//    together with an EA to "test" it's potential, the "potential profit" feature
//    could be very useful for you to test the "potency" of PF charting/trading
//    method. This would help you to choose the best strategy suitable for yourself.
//    For any currency pair and for any TF, you can do some different tests using
//    different TBox and RBox parameters, using a "void" expert, as described here
//    around, and then choose the most "profitable" combination, when you want to
//    do PF trading in real time. Again: Try trading demo first!!!! :D
//    Well, from the theoretical point of view, there was endless discussion if 
//    we have to trade immediately after reversal, or wait for a second box for
//    confirmation. All practical experiments show that waiting for confirmation
//    is much more profitable, prudent and wise. If you just want to buy low 
//    and sell high, forget about Forex! You will lose money. The succes come 
//    (beside of discipline) from buying high and selling back higher, or from
//    selling low and buying lower to cover.... and this would be told to you 
//    by ANY profitable and experienced trader who use a trend-following strategy. 
//    (see trend-folowing on wikipedia and investopedia :P)
//    From our experience, if you just trade after first "other-colored" box, 
//    you will lose all your money fast when the market is ranging and the
//    price is moving one box up, one box down. So, we wait for confirmation first.
//    Of course, this is not the best strategy. For real time trading, you should
//    co-relate the PF signals with other indicators and go "in" only if you have 
//    signal from many of them. You also should trade only when semnificative lines
//    are broken, and not for any PF reversal. In PF theory, you should trade when 
//    prices go out of anterior established ranges, or when they return back to 
//    such kind of ranges. Also, closing a trade is an endless talking. It is 
//    wise to run away at any sign the things could go against you. Cut your loses
//    short! However, some long term investors will not get out at first sign of
//    reversal, but wait until some trend support or resistance is broken (or 
//    major Fibo 38.2 or 23.6 retracement is broken). We are not discussing here
//    which strategy is the best. I am just telling you how do I compute the
//    potential profit, and you certainly can use this in your advantage, for 
//    establishing the best box size, or for whatever you like. I am sure the
//    profit you can realize will be better that the displayed one, as you are
//    a wise trader, taking your sides according with complex money management
//    rules and discipline, and not only following five stupid lines of source 
//    code in an indicator found wherever on the web... So, take care!
//    If the comment line is bothering you or come in conflict with some other
//    indicators or EA you might use, (they would want to display also comments
//    that could be more useful for you, the comment here is not useful in real
//    time, but only when you test is with a void EA), then you can disable the
//    profit computation/show, by changing the ShowPotentialProfit parameter 
//    to false. Unfortunately MT4 does not offer the posibility to display more
//    comments (comming from different indicators) in the same chart, unless you
//    design all the indicators by yourself and use a global variable to exchange
//    the strings. Otherwise, one comment will overwrite another, out of control.

// Q. Why the indicator is only painting the present? Why don't you offer the 
//    painting of the history for free, in the Demo version?
// A. In fact... hmmm... grrr.. I have nothing to offer right now. As I was saying
//    above: a bug in the initial version of the indicator. Even the Full version
//    is not able now to display the past PF charts accurate. This is a "general"
//    disease. All the programs, even "paid ones" from "respected" software houses,
//    that I found "in the wild" up to now, they suffer from the same disease: 
//    they can not paint the PF charts from the past accurate, unless they were
//    computed at that time in the past in real time, stored on broker's servers 
//    and delivered by request same way as candlestick charts are delivered by MT4. 
//    What is the problem with computing the PF charts from the history data? 
//    Well, it is not enough to have the OHLC prices to compute the PF chart. 
//    You must have all the "evolution" of the price "inside" of the candle to be
//    able to paint the PF charting accurate. The alternative is just to use the
//    close price of each candle as "punctual" data, and draw the PF accordingly. 
//    But in this case your PF chart will completely ignore the shadows of the 
//    candles and will miss all the "real" tops and bottoms of the price action. 
//    An alternative is to consider the close price together with high/low prices.
//    But in this case your PF chart will have many reversal points that did not
//    occur in real time (this bug you can see in the russian's guy indicator).
//    To be more clear, and to make you understand what I want to say: imagine we
//    just painted the PF-chart using OHLC prices from a candlestick history chart 
//    until a certain point in the past. We can consider just for the sake of the 
//    example that last painted box was a green one, on the last raising candle 
//    which had a length bigger then our TBox parameter. Now we have to read next
//    candles until we find one that rises higher then current box plus TBox, when
//    we have to paint the next green box, or until we find one that falls lower
//    then current box minus RBox, when we have to reverse and paint the next PF
//    column, with a red box. Quite easy huh? But wait! What the hack is going on?
//    The very next candle is a long one, with the High price much over our CP+TBox 
//    value, AND with the Low price a lot under our CP-RBox value, and no body,
//    or just a very short falling body. What we do? How do we paint the PF boxes?
//    We have some possibilities: 
//     (1) just ignore H/L prices and do nothing (use Close price) - in this case
//         we missed the min/max price action, that can be really important in
//         picking our side for the next tranzaction (was a confirmation there
//         or just a reversal?? or two reversals and two confirmations??)
//     (2) paint only the green box and wait for price to fall until CP-RBox is
//         reached to paint the red box (if occurs later), this would be justified 
//         by the fact that we are in an uptrend already, and the reversal was not
//         confirmed, because the Close price is still very "close" to the Open 
//         price, but in real time we just missed a reversal point that could go
//         against us in a real trade, and could make us lose all the money! 'Coz
//         nobody can guarantee that the price will recover after bumped the Low
//         of the candle.... we should be out of the trade, by far!
//     (3) paint the reversal only - this is even more stupid, we are out of the
//         trade and just lost TBox pips, maybe in real time the price rose first
//         and afterthat falls, so our stop should be TBox pips upper! 
//     (4) paint both the green and red boxes (in which order???) - this is the
//         case for the most of PF charting programs, but this is also far away
//         from being correct, we can have any of the situation described at (2)
//         or (3) above, because we don't know which one occured first in time.
//    This was just the most simple example I could pick. There exists more 
//    complicated situations where painting the next box is a real pain in the 
//    ass.. sorry for the word. You can see some other PF charting solutions
//    (including attached one here in the zip) and conclude for yourself how
//    messy they are. As long as they get the TICKS in real time, everything
//    seems to be OK, but when they take the data from history, the PF chart
//    will look terible: plenty of boxes (up/down sawteeth, etc) should not
//    be there, or min/maxes that should be there are then missing, etc. 
//    To be able to compute the boxes right, one must acces the smaller time 
//    frames, anytime he found a strange situation as the one described above,
//    and get the price evolution with a higher accuracy. The problem with this
//    approach is that the M1 or M5 data are not available for the same period
//    as D1 or W1 data. MT4 has a limitation for how many bars it can store on 
//    the chart. If you have 3000 daily bars, you'll have a 10 years "daily" 
//    history, but for M5 the same amount of history bars will represent only
//    last 10 days of price action, and if you need some M5 data from seven 
//    years ago, then most probably your broker will ask you to pay money
//    for them, and they are HUGE files (M1 data file for one year would have 
//    60*24*365 bars, that is a file of many MB uncompressed). Therefore, the
//    best aproach would be to start generating PF boxes from the present to 
//    the past (time-reversed order) and use M1 data as long as it is available
//    in MT4, then use M5, then M15, and so on until we get what we want (i.e.
//    until we generate the PF chart for our current time frame, that would not
//    be so accurate for ancient time in the past, because no high-resolution
//    data, but will get more and more accurate as we came closer to the 
//    present; anyhow, who is interested to have an accurate chart for 10
//    years ago?! the accuracy should be highest for the "nearby" past, it does
//    not matter that the accuracy is "fading" as we go back in time). In very
//    few cases we still need to make interpolation for the candles that are
//    missing from the higher-resolution charts. This would be the most 
//    accurate aproach, but this would drag us into examining ALL charts for 
//    ALL timeframes lower then the current one, and into making interpolation
//    (same way as MT4 does in the Strategy Tester program)... in a word: a big
//    programming efort. That's why I decided to disable showing the PF chart
//    for the past, and start building it only as long as the server is 
//    "feeding" me with ticks data. Here I can't miss anything.... As I said, 
//    even the Full-Version (which includes the rays and flags, etc) is not able
//    to display the history PF accurate at this moment. I am working to make a
//    new implementation for this, that will probably lead to Version 2.1 of
//    the indicator, and I am still thinking if I will distribute a version
//    "V2.1" of the Demo too. Honestly I see no reason to keep "painting the 
//    history" feature for myself :D. If you disperately need PF charts for 
//    the history straight away, and you are dying NOW to have them, then it 
//    is not so complicate to make them by yourself: you can use the Strategy 
//    Tester to feed you with data and make the interpolation for you. They
//    won't be 100% accurate, as the tester's interpolation is slightly 
//    different from the real time data, but, mainly, ST works in the same way 
//    as I described above, he IS USING the data from smaller TF's to generate
//    ticks to feed the testing EA. You can read more about it, in MT4 help.  
//    So, if you need PF columns for the history, then you have to use MT4's
//    Strategy Tester in "visual mode" and let the M-PF run within a void expert. 
//    To generate a void expert, just right click on the MT4 Navigator window,
//    chose "Create", choose "EA", give it a name (i.e. "Void") and compile it.
//    You will have an expert that does not do anything, but because of that,
//    it is very fast :P. Open any chart with it, check the "visual mode" box,
//    choose "every tick", then hit the "pause" button when it starts to run.
//    Add the M-PF indicator onto the chart (drag it with the mouse from 
//    MT4's Navigator window). Hit the "play" button, give it max speed and go
//    for dinner :P. When you come back, the PF diagram will be ready. If you
//    need it faster (but less accurate, of course), then you can use "control
//    points" or even "open points", or use a smaller TF then the desired one. 
//    Tip: the PF chart will look exactly the same on a smaller TF, as it does
//    not depend on the time, but on the box's vertical size only. So, using a 
//    smaller TF will get the things done faster and more accurate. Of course
//    the "brightness" will look different, because I am counting the candles
//    to decide how much "trading time" passed, so, on smaller TF the boxes 
//    will brighten faster, but you can set the AutoColorSpeed parameter to a 
//    smaller value, or completely ignore the brightness, or turn it off
//    from the AutoColor parameter). Good luck in testing....
//    Note: You can use this method to test any other indicator to see if the 
//    bastard is painting the past or not, so, learn it well! It is quite
//    useful method and I use it every time to test new indicators from web!
//    When the ticks start comming with the speed of the light, you can see 
//    at once if the indicator is repainting it's past, or if something 
//    goes wrong with it. You don't need to design a "specialized" EA (and
//    to use the iCustom function) to test any indicator. Just a "void" EA is
//    the perfect solution. Moreover, you can use the "pause" button and then
//    go step by step through it, using F12 key. 

// Q. Can I get the "every tick" testing mentioned above, done much faster and
//    without needing to go for dinner until the test is finished and the 
//    PF chart is painted? I mean without loosing the accuracy?
// A. Of course you can, but for that you have to work a bit with your mouse
//    and you have to give me a beer if I sell you the pot... Also, you must
//    have a really fast video card in your computer. Metatrader 4 has a
//    little bug in the Strategy Tester program. This in not a bug at all, 
//    but you can call it "feature", if you like. Even if the speed is set to  
//    max (32), he still has to follow some timing rules to paint the incomming 
//    ticks "very fast". The delay does not come from "too less" computing
//    power of your processor, but from the timing implemented in the tester.
//    I mean the delay is longer between ticks if the speed bar is set to 1,
//    and is shorter if the speed is set to 32, but ther is STILL a delay.
//    You can "trick" the ST to make him paint the ticks much faster if you 
//    drag the window on the screen. This is because when the window is 
//    dragged, Windows XP will tell him (send him a message) to repaint all 
//    the content, including the menus, buttons, and of course including the 
//    chart area. This take longer time if you have a "normal" graphic card,
//    therefore the programmers of MT4 did the software in such a way to avoid
//    using any delays when he is parsing the OS messages. If you are Windows 
//    programmer, you know what I mean, this is the normal way. MT4 will repaint
//    the full window at once when he will get "WM_PAINT" message, ignoring 
//    completely the ST's timing. In this case, if you have a FX5200 video 
//    chipset, or other fast graphic card, painting the full window will be
//    faster than the timig of the tester. You have to check the box "show 
//    windows context while dragging" in your Win XP. This is default if you 
//    have a new (fast ot not) video card. In fact, I did not see the other 
//    method (only the frame of the windows is dragged and the content is 
//    re-painted after drop) since many years. Then you must un-maximize your
//    Metatrader program in such a way to cover about 80% of your desktop, by 
//    clicking the "restore down" icon in the upper right corner of MT4 program
//    (most of the people keep MT4 maximized on screen, that is understandable, 
//    as everybody wants to see as much as possible from the charts). Then start
//    the "every tick" testing like above. During the testing, catch the title of
//    your MT4 program's window and drag it with your mouse around the screen. 
//    Amazing huh? If you have a fast graphic card, you will see the ticks
//    "beating" few times faster. Keep dragging it in small circles until the
//    testing is finished. This will take much less time than the version with
//    the dinner... For my laptop is about 5 times faster (ATI chipset) and for
//    the desktop (GeForce chipset) is about 20 times faster.
//    Of course, dinner is funnier... :D 
//    Note: if you have a "normal" video card, you won't be able to see any
//    improvement in speed, or the screen will look "jumpy", painting more  
//    candles in the same time. This is normal and all the computing behind 
//    is still right. For a really good video chipset the movement is "smooth"
//    and REALLY fast.

// Q. What is "AutoBoxHeight" feature, and why should I need it?
// A. I will explain this by another example. We will test our charting
//    method on G/J starting with Jan 2007, until today, August 20. I mean 
//    for all the current year. We will use Demo version, default settings
//    for box sizes (40 and 120), "every tick" for Strategy Tester's 
//    interpolation. You can try doing that parallel with me. This year the 
//    G/J was trending most of the time, that is in my advantage. We would
//    see that our indicator would produce a profit of 2100 pips starting 
//    with 1st of Jan and ending with 13th of April. YES! 2100 pips. Then
//    something is wrong: the market is going ranging. From the end of April 
//    until mid of June we will lose about 1400 pips. All the trends with
//    few exceptions seems to reverse exactly after we took the side. In 
//    July the balance is ... balanced, as we make some small profit, and the
//    big G/J fall from August helps us to recover and get all the money back. 
//    We end the testing on Aug 20, with over 2500 pips profit. That is not 
//    enough to take a holiday, ok, but you should consider that default 
//    values are not the best one, and you can make a better profit by taking 
//    the trades discretionary, not all, according with other indicators too,
//    and at the end of the day, the average 320 pips/month it is NOT that bad  
//    at all, for an "automatic" trading system. Try to repeat this story 
//    for 2005. You will end with a big loss of about 4500 points. Why?! That 
//    is because the market was quite ranging in that period. You still can
//    trade it and make profit, but you will need a different aproach and 
//    a different box size. Now we let our program to compute the box size. 
//    To do that, he is looking to the history data and try to find tops and
//    deeps, to insulate the swings of the price. Like the zig-zag indicator.
//    He is storing all the zig-zags in an array and then sorting them and
//    compute some statistical stuff. He is trying to find the best box size 
//    that would produce maximum profit for the historical data. That would be  
//    the size that allow "as many as possible" X/O-columns to have a length 
//    between 8 and 12 boxes, and as less as possible to have sawteeth (lengths
//    of 2-3-4, that would cause us to lose money if we trade in their direction)
//    and amazing, this values found by experiment, also fit the theory of PF
//    trading! If you select the box size in such a way to have less sawteeth,
//    then you will make less loss. A lenght of 8 for the column allows you to
//    go "in" at the confirmation (second box, you lost 2 boxes from the trend 
//    waiting the confirmation to go in) and allows you to go out at first sign 
//    of reversal (when you lost another 3 boxes because R is about 3*K). 
//    Then you still have 3 boxes left to make profit, i.e. there is also an
//    "error" range, maybe you won't catch all 3 in some shorter columns, but
//    you can catch 2 (i.e. in a 7X-column) or just 1. The upper limit of 12
//    boxes per column ensure us that we will not trade with a box that is
//    too small and be thrown out by the "noises" of the price action. Some 
//    traders even preffer to set the value at 10, but we found that 10 is 
//    sometime too tight and the program will waste a lot of time looking for 
//    the right box-size value to fit the column into 8-10 boxes tall. 
//    Ok, enough with the theory. We also had some attempts in the past to 
//    "optimize" the R/K ratio, but we found that a value between 2 and 4 is 
//    far better then anything else, and we don't do any optimization anymore.
//    I just use 3 and save my time. Trading the 2005 with an "adaptative" box
//    size still made a loss. Well... nobody is perfect :D. But this time the
//    loss is only about 2200pips. Half!! 200% better! And if I repeat the 2007 
//    tests with an auto (adaptative) box size then the result is wonderfull..
//    really amazing: I just produced 4800 pips in 8 months, including the loss
//    of about 550 pips in the "dark" period of April-June. Yes, we still take
//    a loss for that period, my method is not Saint Peter, it lags, it need some
//    time to realize that the market changed. And during this time some trades
//    would "produce" some loss. But 600 pips per month... What a pitty I did not
//    trade this toy in real time long time ago... I have also tested some 
//    variations of this method for long term. With G/U it could produce a
//    small profit of about 700 pips for the last 10 years, with a drawback 
//    not bigger then 3800 pips. It took about 600 trades. Well... You will 
//    laugh loud reading this numbers. I can hear you laughing....
//    "Drawback of thousands of pips? 700 pips profit in 10 years??? You mean
//    70 pips per year? Blearh! That is not something I need"...  
//    If you started laughing, look to the closest wall around you, go there
//    and bump your head against the wall! Three times. And think again! 
//    ALL, but ALL the experts I know, I ever downloaded, tested, read about,  
//    will LOSE ALL the money (CLEAR the account!) if let "unattended" for
//    a long time... There is NO EXPERT available to play with a reasonable
//    risk factor (not 0.000001%, this would not make any profit or lose 
//    bigger then 10 cents per year for a medium account size...), AND to
//    place a reasonable number of trades (not 2 per year with SL at -5pips), 
//    AND to resist 10 years on the market. No one! There ARE experts making
//    money on the market. But they work for short time (months) and need 
//    periodically "adjustments" to keep in touch with the market fashion...
//    As I said, this is not the method to send you in Hawaii. It is just
//    a small step that you can learn if you like to become a profitable 
//    trader. Forex is not easy and you have to work hard and improve 
//    yourself every day.

// Q. Will you ever give us the method to compute the box height... for free?
// A. No. You have to understand that such methods are part of my trading 
//    strategy and are the result of many days and weeks of reading, learning,
//    experimenting, backtesting, and not in the last case, a big programming
//    effort. I am still not decided to sell it. For the time being I will keep
//    it for myself. But now you got the idea, take some books and learn more
//    Technical Analysis and more PNF Theory. You will be able to develop your
//    own trading strategy, and you will be delighted by the progress you made
//    as a trader. And most probably you will be able to compute the box by
//    yourself and make profits with your own strategy. By that time, this
//    method will not be anymore profitable as it is now, and it won't work
//    for me anymore, as the market is continuously changing, and I will also 
//    "evolve" to new and better trading methods and improve myself as a trader,
//    too. That time, to give/sell you my methods that I used in the past (and 
//    I don't use them anymore because they became less profitable for me) would
//    be nothing else then cheating you. And I won't do that... That is why...

// O.K. Now as I typed everything and my fingers are already numb, I would like
// you to give a test to this indicator. Take your favourite pair and time frame
// (even the TF won't matter, except for brightening the boxes). Take the time
// to play with it and use some different sizes for the box and for K/R ratio. 
// After you got the point, relax, stay straight, look at your monitor, and say
// it loud so I could hear you: "Verrry impressive, sir! Thanks" :P
// So I could be happy and sleep well tonight....

#property indicator_chart_window 

//
//---- input parameters 
//
extern int MaxCandlesToCompute=1000; //V2.1++ to speed up the initialization
                                     //For smaller box size and bigger TF, you
                                     //should use smaller param here, or your
                                     //screen could take few minutes to be
                                     //painted first time! (for history painting)
                                     //Use 0 for unlimited (history computed for
                                     //all the chart, not recommended if you 
                                     //use a box of 50 pips on a Daily chart and
                                     //having 5000 bars inside, it could take 
                                     //up to 10 minutes to init the history!)

extern int TrendingBox   = 40;       //this is continuation (K) pattern in pips
                                     //(see the theory of point and figure 
                                     //charting)

extern int ReversingBox  = 120;      //this is reversal (R) pattern size in pips

extern int BoxWidthX     = 2;        //this is the box horizontal width counted in
                                     //bars: 2, 3, 4 and 5 will paint double, 
                                     //triple.. etc.. boxes. All other numbers will
                                     //paint single-candle wide boxes. This feature
                                     //help you to "scale" the boxes on horizontal 
                                     //axis, to have them looking "squares" as in 
                                     //the PF theory when use X and O, because the
                                     //best visual effect is gained when the boxes
                                     //are squares. To make them look squares, 
                                     //you have to corelate the TrendingBox 
                                     //parameter with your time frame and you also
                                     //have to shrink the vertical axis (click and
                                     //drag on the price scale on the right side 
                                     //of your chart) to fit the TrendingBox 
                                     //parameter. If this is not enough and you 
                                     //don't want to change the TF or if you want to
                                     //keep the TrendingBox that you calculated 
                                     //according with your trading strategy, or if 
                                     //you want to see more/less boxes on the screen,
                                     //then you have to alter BoxWidth parameter.
                                     //This is also very useful when you use a lower
                                     //magnification of your chart (more candles 
                                     //on screen, from +/- magnifier buttons will 
                                     //force you to use bigger PF boxes, otherwise
                                     //everything will get confuse on the screen)

extern int AlignBox      = 0;        //this could be any value between 0 and K
                                     //The boxes will be aligned to this value. 
                                     //For example, if you start the indicator
                                     //when the price is 1.2337 and K is 40, 
                                     //maybe you prefer to have boxes aligned
                                     //to 1.2320, 1.2360, 1.2400 and so on, and
                                     //not to 1.2337, 1.2377, 1.2417, etc, as 
                                     //would result from the "naked" calculus.
                                     //By default, AlignBox is 0, so the boxes
                                     //will be aligned to 1.2400, 1.2440, 1.2480,
                                     //1.2520, etc. Sometimes pip-levels matter!
                                     //And some pairs have the bad habit to turn
                                     //around all the time when the last value of
                                     //the price ends with 35, or 85 pips... :P
                                     
extern int VerticalShift = 0;        //this will shift the indicator up or down, 
                                     //if it stays over the candles and you have 
                                     //trouble reading the price on the chart;
                                     // - any positive value (pips) will shift 
                                     // the indicator up (above the graphic, good
                                     // in downtrends when the candles are down)
                                     // - any negative value (pips) will shift 
                                     // the indicator down (below the graphic, 
                                     // good when ... viceversa)
                                     //Tip: you can also set the "chart on
                                     //foregroung" from the chart properties, 
                                     //to hide the indicator's boxes behind the
                                     //candles. In this case could be helpfull
                                     //to have a bigger BoxWidth, otherwise 
                                     //you can miss some rectangles hidden
                                     //behind of the candles 
                                     

extern int HorizontalShift = 1;      //pozitive number for horizontal shift
                                     // - best values are 1 to 5 canddles to the 
                                     // left, as the boxes painted by us 
                                     // would not bother the spectacleof
                                     // the moving prices at the current candle
                                     //(Do NOT use negative numbers!!)
                                     
extern bool AutoColors  = true;      //if true, please use dark colors below
extern color ColorUp    = 0x005500;      // X's are dark green
extern color ColorDown  = 0x000055;      // O's are dark red
extern color ColorFrame = LightSkyBlue;  // borders are... borders
extern int AutoColorSpeed = 7;       //how fast colors change when auto is ON
                                     //(use small pozitive values for steps)

extern color BRayMarkColor  = DeepPink; //color to mark B-Rays
extern color BFlagMarkColor = Yellow;   //color to mark B-Flags
extern int   RayMarkMethod  = 1;        //from 1 to 3, select method, 0 for Off
                                        //any other value would be treated as 1
extern int   RayVerbose     = 1;        //how many rays will be drawn
                                        // 0 - no ray
                                        // 1 - draw all rays (the only method
                                        // implemented up to now, as I have no 
                                        // idea how to select which ray is 
                                        // important and which not)
                                        //Note: this is deleted in Demo version

extern bool  ShowPotentialProfit = true; //display the profit comment for Demo

int MaxObjects;   //to speed up deinit function, 99999 (original) was too much

string Personal;  //add a "secret" number to be able to run multiple 
                  //instances of the indicator on the same chart

int i,direction,index, bw;
double tbxo,tbdraw,k,r,vs;  //some helpful variables

//to make the things crystal clear :D (trend direction parameter)
#define Up 0
#define Dn 1

double ExtBuff[]; //indicator buffer: in Demo used just to test a new candle
                  //for left/right shift; in Full Version returns B/S signals

//just for potential profit computing:
bool isBuying=false, isSelling=false, JustInit=true;
double TotalPips=0, Sold, Bought;
int Tran, MCBars;

//+------------------------------------------+ 
//| Custom indicator initialization function | 
//+------------------------------------------+ 
int init()
{ SetIndexStyle(0,DRAW_NONE);   //not used for painting
  SetIndexBuffer(0,ExtBuff);

  MaxObjects=0; //no object created yet

  k=TrendingBox*Point;          //convert the parameters in a better form
  r=ReversingBox*Point;
  vs=VerticalShift*Point;
  
  //store the "personal" info of the indicator, as the time when it was lounched, 
  //plus a random number, just to avoid instances conflicts (conflicts still 
  //could appear when more instances are started in the same time, as from an expert,
  //that is why we still need to use a random number beside the starting time)
  Personal=TimeToStr(TimeCurrent(),TIME_SECONDS)+"-"+MathRand()+"-";
  
  //compute starting box according with the oldest candle and AlignBox
  //(this will be where the indicator starts)
  if (MaxCandlesToCompute>=10)
    MCBars=MathMin(MaxCandlesToCompute,Bars-100);
  else
    MCBars=Bars-100;  

  int temp=MathMax(MCBars,10);   //must have at least 100 bars
  tbxo=Open[temp]/Point;         //get price as integer (V2.1++)
  //** Demo downgrade: present only! use last 10 bars for taking off
  //tbxo=Open[3]/Point;
  tbxo+=MathMod(AlignBox-tbxo,TrendingBox);  //align it (tricky!)
  tbxo*=Point;                               //and float it back
  
  direction=Up;  //just choose a random direction for the start
  
  //because reversal is usually longer than the continuation, we must
  //center the PF to the price, to avoid planting more O's on bearish
  //reversal, or more X's on bullish reversal. In this way, does not
  //matter how big is R/K ratio, the reversal will be all the 
  //time just one X or one O. Because we started with direction Up, 
  //there will be a vertical shiftup of O's compared with the price: 
  //they will be raised by R-K pips then normal. We repair it here,
  //buy pulling all the graphic down, so both X and O are shifted
  //with the same quantity (X's are down by half, O's are up by half)
  //If sometimes the price appear as playing "out of the box" and a
  //new box is not painted, there is no error or bug. The reason is
  //here, in the alignment and centering of the boxes. 
  tbdraw=NormalizeDouble(tbxo+k-r/2,Digits);

  //compute the box width from the extern parameter
  if (BoxWidthX>=2 && BoxWidthX<=5)
    bw=BoxWidthX;
  else
    bw=1;
  
  index=HorizontalShift+bw;    //where to start? one canddles to the left, 
                               //(should not bother last price spectacle)
  JustInit=true;      //V2.1+, control history shifting
  return (0);
}

//+------------------------------------------------------------------+ 
//| Custom indicator deinitialization function | 
//+------------------------------------------------------------------+ 
int deinit()   //get rid of all objects created by itself
{ for (i=0; i<=MaxObjects; i++)
    { ObjectDelete("Body"+Personal+i);
      ObjectDelete("Frame"+Personal+i);
      ObjectDelete("B-Ray"+Personal+i);
    }
  return(0); 
} 

//+------------------------------------------------------------------+ 
//| Every tick tick tick 
//+------------------------------------------------------------------+ 
int start() 
{ //let's do it in a proper way, we don't need to recalculate milions
  //of bars every time when a tick is comming, that's what the russian
  //guy was doing, making everything VERRYYYY slow...
  int limit;
  int counted_bars=IndicatorCounted();
  //---- check for possible errors
  if(counted_bars<0) return(-1);
  limit=Bars-counted_bars;
  
  //---- main loop (executed only on changed bars, time-ascending order)
  for(int i=limit-1; i>=0; i--)  
  {  if ((i==1 && !JustInit) //ignore uncompleted bar when a newbar is born
                        //if both thicks came in the same time, this would 
                        //improve the accuracy at Open[0] time&price
        || i>=MCBars)   //speed up painting when the user limits the number
    { ExtBuff[i]=0;     //of candles to be computed
      continue;         //.. and mark them as computed
    }
    
    if (ExtBuff[i]==EMPTY_VALUE)  //there is a new "virgin" bar, we have to
    { if (!JustInit) IndexShiftRXO(i); //switch all the boxes to the right 
      ExtBuff[i]=0;               //and then, mark the bar as "spoiled" :P
      
      if (i==0) JustInit=false;   //history painting finish, we can shift
                                  //the boxes to the right now (V2.1+) 
      
      //the idea behind Automatic Color Change is that on a PF chart 
      //some boxes are more important then others. A red box where the 
      //price just drop in 5 minutes making it, is not the same with another
      //red box that took 100 candles and small ocillations to develop and
      //the price banged his head thousand times against the walls of the
      //box, before dropping out of it... In our case, the second box will
      //be painted RED also, same as the first one, but much brighter!!
      
      //Bright boxes show areas of intermediary support and resistance, 
      //congestion zones, can form rays and shows retracements against
      //the main trend (shown by the X or O column where the bright box
      //appears). Almost any "long" column of X or O has some bright boxes
      //inside, and the proportion between them is... guess! Fibonacci.
      
      //Play with the parameters and see for yourself how useful this
      //toy can be. Is up to you hou you use it.
      
      //Ok. Now let's brighten the bars...
      
      if (MaxObjects>0 && AutoColors) //make the bars brighter when the price 
      {                               //bangs his head against box's walls!!
        int Color=ObjectGet("Body"+Personal+MaxObjects,OBJPROP_COLOR);
        int cgrn=(Color&0x00FF00)>>8;
        int cred=(Color&0x0000FF);
        if (cgrn>cred)            //raising trend, add more green, less red
        { int ared=AutoColorSpeed>>1;
          int agrn=AutoColorSpeed|1;
        }
        else                      //falling trend, add more red, less gree
        { agrn=AutoColorSpeed>>1;
          ared=AutoColorSpeed|1;
        }
        
        //blue is the last one, all the time, but we still need some blue, 
        //to make the box become "white-hot" when the price is stuck there
        //for ages, that could show "much stronger" support/resistance
        int ablu=AutoColorSpeed>>2;
        int cblu=(Color&0xFF0000)>>16;
        Color=MathMin((ablu+cblu)<<16,0xFF0000)+  
              MathMin((agrn+cgrn)<<8,0xFF00)+
              MathMin(ared+cred,0xFF);
        ObjectSet("Body"+Personal+MaxObjects,OBJPROP_COLOR, Color);
      }
    }
    
    //actually for Demo we only work with close price, so the test below
    //won't be necessary, but I am lazzy to remove them. We need the H/L 
    //prices only when we play with the history charts
    if (i==0)
    { double hh=Close[0];
      double ll=Close[0];
    }
    else 
    { hh=High[i];
      ll=Low[i];
    }
     
    //well, now let's see what boxes we paint

    if (direction==Up && ll<=tbxo-r)   //is it bearish pullback?
      { IndexShiftLXO();               //start a new O column
        direction=Dn;                  //change the direction 
        CreateXO(tbdraw-k,tbdraw-2*k,ColorDown,Time[i]);  //bearish reversal, 
        tbxo-=r;                       //create new O and update the trend
        ExtBuff[i]=-1.0;
        while (ll <= tbxo-k)           //maybe long jump down? if so, then it is a 
        { CreateXO(tbdraw,tbdraw-k,ColorDown,Time[i]);  //falling trend continuation, 
          tbxo-=k;                     //create new 0 and update the trend
          ExtBuff[i]=-2.0;
        }
      }
    else if (direction==Dn && hh>tbxo+r) //not bearish, is it bullish pull back?
      { IndexShiftLXO();               //start a new X column
        direction=Up;                  //change the direction 
        CreateXO(tbdraw+k,tbdraw+2*k,ColorUp,Time[i]);  //bullish reversal,
        tbxo+=r;                       //create new X and update the trend
        ExtBuff[i]=1.0;
        while (hh >= tbxo+k)           //maybe long jump up? if so, then it is a 
        { CreateXO(tbdraw,tbdraw+k,ColorUp,Time[i]);  //rising trend continuation, 
          tbxo+=k;                     //create new X and update the trend
          ExtBuff[i]=2.0;
        }
     }
    else if (direction==Up && hh>=tbxo+k) //no pullback, is is still bullish? 
    { ExtBuff[i]=2.0;
      while (hh >= tbxo+k)
      { CreateXO(tbdraw,tbdraw+k,ColorUp,Time[i]);  //rising trend continuation, 
        tbxo+=k;                       //create new X and update the trend
      }
    } 
    else if (direction==Dn && ll<=tbxo-k) //or maybe bearish?
    { ExtBuff[i]=-2.0;
      while (ll <= tbxo-k)
      { CreateXO(tbdraw,tbdraw-k,ColorDown,Time[i]);  //falling trend continuation, 
        tbxo-=k;                       //create new 0 and update the trend
      }
    }
    
    //end of story, bye bye! saiyonara!!!!! simpler and more accurate than
    //the russian version
  
  
    // Now let's see if we worth any money for all the job...

    //Do some possible profit computation, if M-PF enters trades after second X
    //or O in a row, and exits at the first sign of reversal (first O or X)
    //(this is usefull ony when the indicator is attached to a visual testing
    //chart and tested with a void EA in visual mode, it will show in comment the
    //possibe profit, realized by "playing" accordig with the strategy above)
    //In "real life" you should not trust this. Use other different additional
    //indicators to help you, and play discretionary! Don't take all trades, 
    //otherwise you will most probably lose money in swinging markets

    //V2.1+ : changes done to compute the potential profit for history too

    if (ShowPotentialProfit)  
    {   double Bid0=Close[i];  //to speed up, ask and bid won't work during testing
        double Ask0=Bid0+MarketInfo(Symbol(),MODE_SPREAD)*Point;
        if (ExtBuff[i]>0.0 && isSelling)   //stop selling
        { isSelling=false;
          TotalPips=TotalPips+(Sold-Ask0)/Point;
          Sold=0;
        }
        if (ExtBuff[i]<0.0 && isBuying)    //stop buying
        { isBuying=false;
          TotalPips=TotalPips+(Bid0-Bought)/Point;
          Bought=0;
        }
        if (ExtBuff[i]>1.0 && !isBuying)    //bullish confirmation, must buy
        { isBuying=true;
          Bought=Ask0;
          Tran++;
        }
        if (ExtBuff[i]<-1.0 && !isSelling)  //bearish confirmation, must sell
        { isSelling=true;
          Sold=Bid0;
          Tran++;
        }
        Comment("Tranzaction ",Tran,": -> Selling: ",isSelling," at ",Sold,
             ", Buying: ",isBuying," at ",Bought,", Profit: ",TotalPips);
    }
  }  //end "for all uncounted bars" (changed in V2.1+ to include the profit 
     //calculation also for the history chart, but this is not so accurate.
     //This is because long candles would be painted "at once" in the loop
     //(all boxes for whole the candle), and the "entry" after second box
     //will be missed. Therefore the displayed profit for the history is 
     //all the time lower than the real profit following the "strategy". 
     //Anyhow, profit calculus is just "a sketch", due to the fact that a 
     //wise trader would never take ALL the trades according with the 
     //"strategy" used here to compute the potential profit.
     
  return(0);
}  

//*************************************************************** 
// Additional function that creates a new X or new O box
//*************************************************************** 
void CreateXO(double from, double to, double Color, datetime di)
{  MaxObjects++;

   ObjectCreate("Body"+Personal+MaxObjects,OBJ_RECTANGLE,0,
      Time[index],from+vs,Time[index-bw],to+vs);
   ObjectSet("Body"+Personal+MaxObjects,OBJPROP_STYLE, STYLE_SOLID);
   ObjectSet("Body"+Personal+MaxObjects,OBJPROP_BACK,  True);
   ObjectSet("Body"+Personal+MaxObjects,OBJPROP_COLOR, Color);
   ObjectSetText("Body"+Personal+MaxObjects,"CandleTime: "+TimeToStr(di));

   ObjectCreate("Frame"+Personal+MaxObjects,OBJ_RECTANGLE,0,
      Time[index],from+vs,Time[index-bw],to+vs);
   ObjectSet("Frame"+Personal+MaxObjects,OBJPROP_STYLE, STYLE_SOLID);
   ObjectSet("Frame"+Personal+MaxObjects,OBJPROP_BACK,  False);
   ObjectSet("Frame"+Personal+MaxObjects,OBJPROP_COLOR, ColorFrame);

   tbdraw=to;
} 

//*************************************************************** 
// Additional function that shifts all bars to the right when a 
// new candle is forming, to avoid left-running effect
//*************************************************************** 
void IndexShiftRXO(bool history)
{ for (int j=1; j<=MaxObjects; j++)
  { //just fore more clarity, and solve infinite-bars problem when painting
    //and coloring the history boxes:
    datetime dBody1 =ObjectGet("Body" +Personal+j,OBJPROP_TIME1);
    if (GetLastError()==4202 && history)
       continue; //v2.1++ solve infinite-sliding bug (if TF is changed on the fly)
    datetime dBody2 =ObjectGet("Body" +Personal+j,OBJPROP_TIME2);
    if (GetLastError()==4202 && history)
       continue; //v2.1++ solve infinite-sliding bug (idem)
    datetime dFrame1=ObjectGet("Frame"+Personal+j,OBJPROP_TIME1);
    if (GetLastError()==4202 && history)
       continue; //v2.1++ solve infinite-sliding bug (idem)
    datetime dFrame2=ObjectGet("Frame"+Personal+j,OBJPROP_TIME2);
    if (GetLastError()==4202 && history)
       continue; //v2.1++ solve infinite-sliding bug (idem)
    datetime dRay1  =ObjectGet("B-Ray"+Personal+j,OBJPROP_TIME1);
    if (GetLastError()==4202 && history)
       continue; //v2.1++ solve infinite-sliding bug (idem)
    datetime dRay2  =ObjectGet("B-Ray"+Personal+j,OBJPROP_TIME2);
    if (GetLastError()==4202 && history)
       continue; //v2.1++ solve infinite-sliding bug (idem)

    ObjectSet("Body"+Personal+j, OBJPROP_TIME1,Time[iBarShift(Symbol(),0,dBody1)-1]);
    ObjectSet("Body"+Personal+j, OBJPROP_TIME2,Time[iBarShift(Symbol(),0,dBody2)-1]);
    ObjectSet("Frame"+Personal+j,OBJPROP_TIME1,Time[iBarShift(Symbol(),0,dFrame1)-1]);
    ObjectSet("Frame"+Personal+j,OBJPROP_TIME2,Time[iBarShift(Symbol(),0,dFrame2)-1]);

    //we let the moving of the rays here for the Demo version and won't
    //remove them. Maybe the user wants to draw his own rays and rename them
    //to "B-Ray" naming scheme, to have them attached to the PF chart and
    //to avoid having them lost to the left with the candlestick chart
    ObjectSet("B-Ray"+Personal+j,OBJPROP_TIME1,Time[iBarShift(Symbol(),0,dRay1)-1]);
    ObjectSet("B-Ray"+Personal+j,OBJPROP_TIME2,Time[iBarShift(Symbol(),0,dRay2)-1]);
  }
}

//*************************************************************** 
// Additional function that shifts all bars to the left when a 
// new column of X's or O's is forming
//*************************************************************** 
void IndexShiftLXO()
{ for (int j=0; j<=MaxObjects; j++)
  { //just fore more clarity:
    datetime dBody1 =ObjectGet("Body" +Personal+j,OBJPROP_TIME1);
    datetime dBody2 =ObjectGet("Body" +Personal+j,OBJPROP_TIME2);
    datetime dFrame1=ObjectGet("Frame"+Personal+j,OBJPROP_TIME1);
    datetime dFrame2=ObjectGet("Frame"+Personal+j,OBJPROP_TIME2);
    datetime dRay1  =ObjectGet("B-Ray"+Personal+j,OBJPROP_TIME1);
    datetime dRay2  =ObjectGet("B-Ray"+Personal+j,OBJPROP_TIME2);
    
    ObjectSet("Body"+Personal+j, OBJPROP_TIME1,Time[iBarShift(Symbol(),0,dBody1)+bw]);
    ObjectSet("Body"+Personal+j, OBJPROP_TIME2,Time[iBarShift(Symbol(),0,dBody2)+bw]);
    ObjectSet("Frame"+Personal+j,OBJPROP_TIME1,Time[iBarShift(Symbol(),0,dFrame1)+bw]);
    ObjectSet("Frame"+Personal+j,OBJPROP_TIME2,Time[iBarShift(Symbol(),0,dFrame2)+bw]);

    //we need to move the B-rays too, see the comment above
    ObjectSet("B-Ray"+Personal+j,OBJPROP_TIME1,Time[iBarShift(Symbol(),0,dRay1)+bw]);
    ObjectSet("B-Ray"+Personal+j,OBJPROP_TIME2,Time[iBarShift(Symbol(),0,dRay2)+bw]);
  }
}

 